<?php

if ( !class_exists('wsPluginVoter') ):

class wsPluginVoter {
	
	var $username = ''; //WordPress.org username
	var $password = ''; //WordPress.org password
	var $logged_in = false; //Whether the current connection is already logged in
	
	var $connection; //Current connection resource.
	
  /**
   * Class constructor.
   *
   * @param string $username WordPress.org account name
   * @param string $password WordPress.org account password 
   * @return void
   */
	function wsPluginVoter($username = '', $password = ''){
		$this->username = $username;
		$this->password = $password;
		$this->connection = null;
	}
	
  /**
   * Initialize and configure a CURL handle.
   * If CURL has already been initialized the cached handle will be returned. 
   *
   * @return resource|WP_Error A valid CURL resource handle or an instance of WP_Error if something went wrong.
   */
	function init_connection(){
		if ( is_resource($this->connection) ){
			//CURL connection already initialized
			return $this->connection;
		}
		
		if ( !function_exists('curl_exec') ){
			return new WP_Error(
				'curl_not_installed',
				'CURL not installed'
			);
		}
		
		$this->connection = curl_init();
		if ( !is_resource($this->connection) ){
			return new WP_Error(
				'curl_init_failure',
				"Can't initialize CURL"
			);
		}
		
        curl_setopt($this->connection, CURLOPT_RETURNTRANSFER, true);
        curl_setopt($this->connection, CURLOPT_TIMEOUT, 60);
        
        //Masquerade
        $ua = 'Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 5.1; .NET CLR 1.1.4322; .NET CLR 2.0.50727; .NET CLR 3.0.04506.30)';
        curl_setopt($this->connection, CURLOPT_USERAGENT, $ua);
        
        @curl_setopt($this->connection, CURLOPT_FOLLOWLOCATION, true);
        curl_setopt($this->connection, CURLOPT_MAXREDIRS, 10);
        curl_setopt($this->connection, CURLOPT_FAILONERROR, true);
        
        //A cookie file is required to store the WP.org login cookie
        $cookie_file = $this->get_cookie_file_name();
        if ( empty($cookie_file) ){
			return new WP_Error(
				'no_cookie_file',
				"Can't create a temporary file for cookies"
			);
		}
		
		//Don't store cookies between sessions. It would be insecure.
		if ( file_exists($cookie_file) ){
			unlink($cookie_file);
		}
		
        curl_setopt($this->connection, CURLOPT_COOKIEFILE, $cookie_file);
        curl_setopt($this->connection, CURLOPT_COOKIEJAR, $cookie_file);
        
        return $this->connection;
	}
	
  /**
   * Get the name of a temporary file that can be used for cookie storage.
   *
   * @return string
   */
	function get_cookie_file_name(){
		return trailingslashit($this->get_tmp_directory()) . 'wp_vfp_cookies';
	}
	
  /**
   * Automagically choose & return a directory suitable for the storage of temporary files.  
   *
   * @return string|bool Full directory path, or FALSE if a temp. directory can't be found.
   */
	function get_tmp_directory(){
		//Try to get the system-wide temp. dir. first
		if ( function_exists('sys_get_temp_dir') ){
			$directory = sys_get_temp_dir();
			if ( $directory && @is_writable($directory)){
				return $directory; 
			}
		}
		
		//Try the default /tmp
		$directory = '/tmp';
		if ( @is_writable($directory) ){
			return $directory;
		}
		
		//Try the upload directory.  
		$directory = ini_get('upload_tmp_dir');
		if ( $directory && @is_writable($directory)){
			return $directory;
		}
		
		//Try the plugin's own directory.
		$directory = dirname(__FILE__);
		if ( @is_writable( $directory ) ){
			return $directory;
		}
		
		return false;
	}
	
  /**
   * Clean up any external resources used by this class. This method should be called
   * once you're finished your voting/data extraction and are about to discard this object.
   *
   * @return void
   */
	function done(){
		//Close the CURL handle
		curl_close($this->connection);
		
		$this->connection = null;
		$this->logged_in = false;
		
		//Smash the cookiejar. Leaving the file behind would be dangerous as someone 
		//could use the stored cookies to access the user's wordpress.org account.  
		$cookie_file = $this->get_cookie_file_name();
		if ( file_exists($cookie_file) && @is_writable($cookie_file) ){
			unlink($cookie_file);
		}
	}
	
  /**
   * Log in to WordPress.org.
   *
   * This method does nothing if we're already logged in.
   *
   * @return bool|WP_Error
   */
	function log_in(){
		if ( $this->logged_in ){
			return true;
		}
		
		$conn = $this->init_connection();
		if ( is_wp_error($conn) ){
			return $conn;
		}
		
		$login_form_action = 'http://wordpress.org/extend/plugins/bb-login.php';
		curl_setopt($conn, CURLOPT_URL, $login_form_action);
		curl_setopt($conn, CURLOPT_POST, true);
		
		$params = array(
			'user_login' => $this->username,
			'password' => $this->password,
			'Submit' => 'Log in',
			're' => '',
		);
		curl_setopt($conn, CURLOPT_POSTFIELDS, build_query($params));
		
		$result = curl_exec($conn);
		
		/*
		After a successful login, the user is redirected to a page that contains the string
		"Welcome, username!" and a link to the user's profile. We use this knowledge to check
		if we managed to log in okay. 
		*/
		$profile_url = 'http://wordpress.org/extend/plugins/profile/' . urlencode($this->username);
		$login_ok = (strpos($result, 'Welcome,') !== false) && (strpos($result, $profile_url) !== false);
		
		if ( $login_ok ){
			$this->logged_in = true;
			return true;
		} else {
			$this->logged_in = false;
			return new WP_Error(
				'login_failed',
				"Couldn't login to WordPress.org"
			);
		}
	}
	
  /**
   * Vote about a plugin's compatibility with a specific WP version.
   *
   * @param string $plugin
   * @param bool $compatible
   * @param string $plugin_version
   * @param string $wordpress_version
   * @return bool|WP_Error
   */
	function vote($plugin, $compatible, $plugin_version, $wordpress_version){
		//Login to wordpress.org
		$login = $this->log_in();
		if ( is_wp_error($login) ){
			return $login;
		}
		
		$conn = $this->connection;
		
		//Load the plugin's directory page
		$html = $this->fetch_directory_page($plugin);
		if ( is_wp_error($html) ){
			return $html;
		}
		
		//Extract the form fields that we'll need to send the vote.
		$fields = array();
		if ( preg_match('@<form[^<>]+class="compatibility"[^<>]*>(?P<form_contents>.+?)</form>@si', $html, $matches) ){
			$inputs = $this->extract_tags($matches['form_contents'], 'input', true);
			foreach($inputs as $input){
				if ( !isset($input['attributes']['name']) ) {
					continue; //Skip unnamed fields
				}
				
				$fields[$input['attributes']['name']] = isset($input['attributes']['value'])?$input['attributes']['value']:'';
			}
		} else {
			return new WP_Error(
				'unknown_page_format',
				"Can't parse the directory page of this plugin."
			);
		}
		
		//Prepare the voting data.
		$fields['compatibility[version]'] = $wordpress_version;
		$fields['compatibility[topic_version]'] = $plugin_version;
		
		//"Works" = 1, "Broken" = 0
		$compatible = $compatible?1:0;
		$fields['compatibility[compatible]'] = $compatible;
		
		//Submit the vote
		curl_setopt($conn, CURLOPT_POST, true);
		curl_setopt($conn, CURLOPT_POSTFIELDS, build_query($fields));
		$html = curl_exec($conn);
		
		if ( empty($html) ){
			return new WP_Error(
				'http_request_failed',
				"Vote submission failed mysteriously."
			);
		}
		
		//Verify that the vote was registered. If it was, the appropriate form button 
		//will now have the "active" class.  
		$buttons = $this->extract_tags($html, 'button', false);
		foreach($buttons as $button){
			if ( !isset($button['attributes']['name']) || ($button['attributes']['name'] != 'compatibility[compatible]') ){
				continue;
			}
			
			if ( intval($button['attributes']['value']) == $compatible ){
				$classes = explode(' ', $button['attributes']['class']);
				if ( in_array('active', $classes) ){
					return true;
				} else {
					return new WP_Error(
						'voting_failed',
						"Your vote was not registered. Perhaps the voting process has changed."
					);
				}
			}
		}
		
		return new WP_Error(
			'voting_failed',
			"Your vote was not registered. Perhaps the voting process has changed."
		);
	}
	
  /**
   * Submit a star rating to the wordpress.org directory. 
   *
   * @param string $plugin Plugin file.
   * @param int $rating Rating on the scale of 1 to 5 stars. 
   * @return bool|WP_Error True on success, or an istance of WP_Error on failure.
   */
	function rate($plugin, $rating){
		//Login to wordpress.org
		$login = $this->log_in();
		if ( is_wp_error($login) ){
			return $login;
		}
		
		$conn = $this->connection;
		
		//Load the plugin's directory page
		$html = $this->fetch_directory_page($plugin);
		if ( is_wp_error($html) ){
			return $html;
		}
		
		//Extract the "topic id" of this plugin
		if ( preg_match('@rate-topic_(?P<topic_id>\d+)@i', $html, $matches) ){
			$topic_id = intval($matches['topic_id']);
		} else {
			return new WP_Error(
				'topic_id_missing',
				"Couldn't find the plugin ID on the WordPress.org page"
			);
		}
		
		$rating_url = 'http://wordpress.org/extend/plugins/bb-admin/admin-ajax.php';
		
		//Submit the rating
		$conn = $this->init_connection();
	
		curl_setopt($conn, CURLOPT_URL, $rating_url);
		curl_setopt($conn, CURLOPT_POST, true);
		curl_setopt($conn, CURLOPT_REFERER, $this->get_plugin_directory_url($plugin));
		
		$params = array(
			'cookie' => '', //Sent but ignored.
			'action' => 'rate-topic',
			'id' => $topic_id,
			'rate' => $rating,
			'_' => '',
		);
		$post = http_build_query($params, null, '&');
		curl_setopt($conn, CURLOPT_POSTFIELDS, $post);
		
		$html = curl_exec($conn);
		
		//In some (which?) cases, the AJAX handler will simply return "0" to signify an error.
		if ( $html === '0' ){
			return new WP_Error(
				'rating_failed', 
				"The rating was refused by WordPress.org."
			);
		}
		
		//On success, the response will be a small XML document that, among other stuff,
		//contains the <wp_ajax> element. Note : this may not be sufficient to verify if
		//the rating was successfully sent.
		if ( strpos($html, '<wp_ajax>') !== false ){
			return true;
		} else {
			return new WP_Error(
				'invalid_response',
				"Received an unrecognized response from WordPress.org"
			);
		}
	}
	
  /**
   * Load the plugin's description page from the WordPress.org plugin directory.
   *
   * @param string $plugin Plugin filename.
   * @return string|WP_Error Either the page contents, or an instance of WP_Error if something went wrong.
   */
	function fetch_directory_page($plugin){
		$conn = $this->init_connection();
		
		//Load the plugin's directory page
		$plugin_page = $this->get_plugin_directory_url($plugin);
		
		curl_setopt($conn, CURLOPT_URL, $plugin_page);
		curl_setopt($conn, CURLOPT_HTTPGET, true);
		$html = curl_exec($conn);
		
		if ( empty($html) ){
			return new WP_Error(
				'http_request_failed',
				"Couldn't load the plugin's directory page."
			);
		}
		
		//If the plugin isn't listed, we'll get redirected to the directory homepage.
		if (curl_getinfo($conn, CURLINFO_EFFECTIVE_URL) == 'http://wordpress.org/extend/plugins/'){
			return new WP_Error(
				'not_in_directory',
				"This plugin is not listed in the WordPress.org directory, so you can't vote for it."
			);
		};
		
		return $html;
	}
	
  /**
   * Try to guess the plugin's wordpress.org URL from a its filename.
   *
   * @param string $plugin Plugin file, e.g. "cool-widget/cool-widget.php"
   * @return string
   */
	function get_plugin_directory_url($plugin){
		$plugin_slug = dirname(plugin_basename($plugin)); //May fail if the plugin dir. is renamed.
		if ( empty($plugin) || ($plugin_slug == '.') ){
			$plugin_slug = str_replace('.php', '', basename($plugin));
		}
		$plugin_page = 'http://wordpress.org/extend/plugins/' . urlencode($plugin_slug) . '/';
		return $plugin_page;
	}
	
  /**
   * Extract plugin compatibility votes & star ratings from the HTML of its wp.org directory page.
   *
   * @param string $html
   * @return array Associative array with these keys : 'all_votes', 'user_votes', 'average_rating', 'user_rating'
   */
	function extract_plugin_votes($html){
		@$dom = DOMDocument::loadHTML($html);
 		$xpath = new DOMXpath($dom);
		
		//Find WP version rows in the compatibility table 
		$wp_version_rows = $xpath->query( '//table[@id = "compatibility-table"]/tr' );
		 
		$all_votes = array();
		$user_votes = array();
		
		//Extract votes
		foreach($wp_version_rows as $wp_version_row){
			//WordPress version is encoded in the row ID, e.g. "compatibility-version-2_9_2".
			$id = $wp_version_row->getAttribute('id');
			if ( empty($id) ){
				continue;
			}
			$wp_version = str_replace('compatibility-version-', '', $id);
			$wp_version = trim(str_replace('_', '.', $wp_version));
			
			$all_votes[$wp_version] = array();
			$user_votes[$wp_version] = array();
			
			//Extract votes for each WP version/plugin version combination
			$plugin_version_cells = $wp_version_row->getElementsByTagName('td');
			foreach($plugin_version_cells as $plugin_version_cell){
				//Plugin version is encoded in the "class" attribute, e.g. "compatibility-topic-version-1_0".
				$matched = preg_match(
					'@compatibility-topic-version-(?P<version_number>[^\s]+?)(?:\s|$)@i', 
					$plugin_version_cell->getAttribute('class'),
					$matches 
				);
				
				if ( !$matched ) {
					continue;
				}
				$plugin_version = trim( str_replace('_', '.', $matches['version_number']));
				
				//Parse the vote counts. The format is "percentage,total,works"
				if ( !isset($plugin_version_cell->textContent) ){
					continue;
				}
				$vote_data = explode(',', $plugin_version_cell->textContent);
				if ( count($vote_data) != 3 ){
					continue; //Skip rows with unknown format
				}
				
				$vote_data = array(
					'total_votes' => intval($vote_data[1]),
					'works' => intval($vote_data[2]),
					'percentage_works' => intval($vote_data[0]),
				);
				$all_votes[$wp_version][$plugin_version] = $vote_data;
				
				//The current user's vote may also be present in the classnames. If the user
				//has marked the plugin as working, the "class" attr. will include the "user-works"
				//value. If they marked it as broken, it will be "user-broken" instead.				
				$matched = preg_match(
					'@(?:\s|^)user-(?P<user_vote>works|broken)(?:\s|$)@',
					$plugin_version_cell->getAttribute('class'),
					$matches
				);
				if ( $matched ){
					switch($matches['user_vote']){
						case 'works':
							$user_votes[$wp_version][$plugin_version] = true;
							break;
						case 'broken':
							$user_votes[$wp_version][$plugin_version] = false;
							break;	
					}
				}
			}
		}
		
		//Extract star ratings.
		//The result will be a floating point number between 0 and 100.
		//Divide by 5 to get the number of "stars". 
		$average_rating = $user_rating = null;
		$ratings = $xpath->query('//div[@id="pagebody"]//div[contains(@class, "star-rating")]');
		if ( $ratings->length >= 1 ){
			$style = $ratings->item(0)->getAttribute('style');
			if ( preg_match('@width\s*:\s*(?P<pixels>[\d.]+)px\s*(?:;|$)@i', $style, $matches) ){
				$average_rating = floatval($matches['pixels']);
			}
		}
		if ( $ratings->length >= 2 ){
			$style = $ratings->item(1)->getAttribute('style');
			if ( preg_match('@width\s*:\s*(?P<pixels>[\d.]+)px\s*(?:;|$)@i', $style, $matches) ){
				$user_rating = floatval($matches['pixels']);
			}
		}	
		
		return array(
			'all_votes' => $all_votes,
			'user_votes' => $user_votes,
			'average_rating' => $average_rating,
			'user_rating' => $user_rating,
		);
	}
	
	/**
	 * extract_tags()
	 * Extract specific HTML tags and their attributes from a string.
	 *
	 * You can either specify one tag, an array of tag names, or a regular expression that matches the tag name(s). 
	 * If multiple tags are specified you must also set the $selfclosing parameter and it must be the same for 
	 * all specified tags (so you can't extract both normal and self-closing tags in one go).
	 * 
	 * The function returns a numerically indexed array of extracted tags. Each entry is an associative array
	 * with these keys :
	 * 	tag_name	- the name of the extracted tag, e.g. "a" or "img".
	 *	offset		- the numberic offset of the first character of the tag within the HTML source.
	 *	contents	- the inner HTML of the tag. This is always empty for self-closing tags.
	 *	attributes	- a name -> value array of the tag's attributes, or an empty array if the tag has none.
	 *	full_tag	- the entire matched tag, e.g. '<a href="http://example.com">example.com</a>'. This key 
	 *		          will only be present if you set $return_the_entire_tag to true.	   
	 *
	 * @param string $html The HTML code to search for tags.
	 * @param string|array $tag The tag(s) to extract.							 
	 * @param bool $selfclosing	Whether the tag is self-closing or not. Setting it to null will force the script to try and make an educated guess. 
	 * @param bool $return_the_entire_tag Return the entire matched tag in 'full_tag' key of the results array.  
	 * @param string $charset The character set of the HTML code. Defaults to ISO-8859-1.
	 *
	 * @return array An array of extracted tags, or an empty array if no matching tags were found. 
	 */
	function extract_tags( $html, $tag, $selfclosing = null, $return_the_entire_tag = false, $charset = 'ISO-8859-1' ){
	 
		if ( is_array($tag) ){
			$tag = implode('|', $tag);
		}
	 
		//If the user didn't specify if $tag is a self-closing tag we try to auto-detect it
		//by checking against a list of known self-closing tags.
		$selfclosing_tags = array( 'area', 'base', 'basefont', 'br', 'hr', 'input', 'img', 'link', 'meta', 'col', 'param' );
		if ( is_null($selfclosing) ){
			$selfclosing = in_array( $tag, $selfclosing_tags );
		}
	 
		//The regexp is different for normal and self-closing tags because I can't figure out 
		//how to make a sufficiently robust unified one.
		if ( $selfclosing ){
			$tag_pattern = 
				'@<(?P<tag>'.$tag.')			# <tag
				(?P<attributes>\s[^>]+)?		# attributes, if any
				\s*/?>							# /> or just >, being lenient here 
				@xsi';
		} else {
			$tag_pattern = 
				'@<(?P<tag>'.$tag.')			# <tag
				(?P<attributes>\s[^>]+)?		# attributes, if any
				\s*>							# >
				(?P<contents>.*?)				# tag contents
				</(?P=tag)>						# the closing </tag>
				@xsi';
		}
	 
		$attribute_pattern = 
			'@
			(?P<name>\w+)											# attribute name
			\s*=\s*
			(
				(?P<quote>[\"\'])(?P<value_quoted>.*?)(?P=quote)	# a quoted value
				|							# or
				(?P<value_unquoted>[^\s"\']+?)(?:\s+|$)				# an unquoted value (terminated by whitespace or EOF) 
			)
			@xsi';
	 
		//Find all tags 
		if ( !preg_match_all($tag_pattern, $html, $matches, PREG_SET_ORDER | PREG_OFFSET_CAPTURE ) ){
			//Return an empty array if we didn't find anything
			return array();
		}
	 
		$tags = array();
		foreach ($matches as $match){
	 
			//Parse tag attributes, if any
			$attributes = array();
			if ( !empty($match['attributes'][0]) ){ 
	 
				if ( preg_match_all( $attribute_pattern, $match['attributes'][0], $attribute_data, PREG_SET_ORDER ) ){
					//Turn the attribute data into a name->value array
					foreach($attribute_data as $attr){
						if( !empty($attr['value_quoted']) ){
							$value = $attr['value_quoted'];
						} else if( !empty($attr['value_unquoted']) ){
							$value = $attr['value_unquoted'];
						} else {
							$value = '';
						}
	 
						//Passing the value through html_entity_decode is handy when you want
						//to extract link URLs or something like that. You might want to remove
						//or modify this call if it doesn't fit your situation.
						$value = html_entity_decode( $value, ENT_QUOTES, $charset );
	 
						$attributes[$attr['name']] = $value;
					}
				}
	 
			}
	 
			$tag = array(
				'tag_name' => $match['tag'][0],
				'offset' => $match[0][1], 
				'contents' => !empty($match['contents'])?$match['contents'][0]:'', //empty for self-closing tags
				'attributes' => $attributes, 
			);
			if ( $return_the_entire_tag ){
				$tag['full_tag'] = $match[0][0]; 			
			}
	 
			$tags[] = $tag;
		}
	 
		return $tags;
	}
}

endif;

?>